import requests
import os
from pathlib import Path
from concurrent.futures import ThreadPoolExecutor, as_completed
import logging
import time
import threading
import math

# Global counters
total_downloads = 0
completed_downloads = 0
download_status = {}
download_speeds = {}

def update_completed_downloads(downloads, folder):
    global completed_downloads
    for _, file_name in downloads:
        if check_downloaded(file_name, folder):
            completed_downloads += 1

def setup_logging():
    logging.basicConfig(filename='download_log.log', level=logging.INFO, 
                        format='%(asctime)s %(levelname)s: %(message)s', 
                        datefmt='%Y-%m-%d %H:%M:%S')

def check_downloaded(file_name, folder):
    log_exists = False
    file_path = Path(folder) / file_name
    try:
        with open('download_log.log', 'r') as log_file:
            for line in log_file:
                if file_name in line and "Successfully downloaded" in line:
                    log_exists = True
                    break
    except FileNotFoundError:
        log_exists = False

    return log_exists and file_path.exists()

def human_readable_size(size_in_bytes):
    if size_in_bytes == 0:
        return '0B'
    size_name = ('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB')
    i = int(math.floor(math.log(size_in_bytes, 1024)))
    p = math.pow(1024, i)
    size = round(size_in_bytes / p, 2)
    return f'{size} {size_name[i]}'

headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'en-US,en;q=0.9,tr;q=0.8',
    'Cache-Control': 'no-cache',
    'Pragma': 'no-cache',
    'Sec-Ch-Ua': '"Not_A Brand";v="8", "Chromium";v="120", "Google Chrome";v="120"',
    'Sec-Ch-Ua-Mobile': '?0',
    'Sec-Ch-Ua-Platform': '"Windows"',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36'
}
    
cookies = {
'__Secure-civitai-token': 'eyJhbGciOiJkaXIiLCJlbmMiOiJBMjU2R0NNIn0..Q1UneZNuLjdQtraV.mx3JOJszf5FktzJVb1Fj4RKIe6jLItRnldntOZTThl0P28rOpIPfguy-dIWohoAfW43gEO-ngZKt_i5u2uZVvNt6SiIJ2HeasITQ10id5EshQ73xuj8kanQLJpSBV6O1n1kb-MOte6agsH99nqdlK7si6viAeONl1-OD5L57XtNrwtHCz73Ucc6gBLKeq_uaxVbhvbTl0u13wOGTVVX8Z1_AFO99HuhM1XBhCM9iZ8F4PAbqpDUSo1hIDxA8duT5XHVB_JM9ZwWPOtWoN-OcCPVVSntVhBLCqNX4wlBEWvqIOOKuCnBw_6D64-5M-J2lZABufDOR0sxeM357T_qpx4IqRzB1xf4v8aF5K_I8bDowXpsvf-S5Ik9wYATC6F2NCDBX0a6msnw043JSOnW2VaSCLvTI_6YZdjrOMaFv-UfegUn9okiraIqFnOxFCWK7Fh6hUv0gxLqrdOMp224dGNzLjFX97DcUQlEdwwW1kNFdS9dYltsXFaWYGmjZPJlz4RrU3V0sHGzsc8oCTJ8-Z_ZAEkMFYlxmqU81cz01cNDWFKFax6dVtM9Dkvd8yP5Unt0343NN7peolSST_H4mSth0a2SnLZUd23s1eVSqGJ1S_sFS25PS6pCmOxvevzb5B8-nwqnPk07nobLVrwfbBhTtE8uc8o5XgHGZRsP9c2GBC_t5MWCXnSe4TpKFYB9DmeIo4udDPB13WM3zCa48rOZUlZAn3ZovBBkgVGYi94dhb1waG2sresyELLn68UORGBKXkf1ZgRbcLo8qa_ulWEYSlzANtrNOBfgJs2cIPyPMmS9-3JrOuTQq8cu4a43iiu_U9jXAiGXqklqXmGYUF5CE8eKY5-4k8zd84VfM8t_QCZ4kLml1QwhYSkXFZq4pyDhc-_6U6Z9A-g0NST6ZkLkoiAJWLs8ZnaU.fYwch25qytTeExXN0v1Fsg; Domain=.civitai.com; Path=/; Expires=Fri, 16 Feb 2024 01:39:33 GMT; HttpOnly; Secure; SameSite=Lax'
}


def download_file(url, folder, file_name):
    global completed_downloads, download_status, download_speeds

    if check_downloaded(file_name, folder):
        print_progress()
        print(f"File {file_name} already downloaded and exists. Skipping download.")
        download_status[file_name] = 'Completed'
        return

    folder_path = Path(folder)
    folder_path.mkdir(parents=True, exist_ok=True)
    file_path = folder_path / file_name

    try:
        response = requests.get(url, headers=headers, cookies=cookies, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024
        lock = threading.Lock()
        progress = 0
        start_time = time.time()

        def print_speed():
            nonlocal progress
            while progress < total_size:
                time.sleep(5)
                with lock:
                    elapsed_time = time.time() - start_time
                    downloaded_size = progress
                speed = (downloaded_size / (1024 * 1024)) / elapsed_time if elapsed_time > 0 else 0
                downloaded_size_gb = human_readable_size(downloaded_size)
                total_size_gb = human_readable_size(total_size)
                status = download_status.get(file_name, 'Downloading...')
                print(f"{file_name} \t\t\t {speed:.2f} MB/s, {downloaded_size_gb} / {total_size_gb}, {status}")

        threading.Thread(target=print_speed, daemon=True).start()

        with open(file_path, 'wb') as file:
            for data in response.iter_content(block_size):
                file.write(data)
                with lock:
                    progress += len(data)
                download_status[file_name] = f'Downloading... {progress / total_size * 100:.2f}%'
                elapsed_time = time.time() - start_time
                current_speed = progress / elapsed_time / (1024 * 1024)  # Calculate current speed in MB/s
                download_speeds[file_name] = current_speed  # Update the speed in global dictionary

            if total_size != 0 and progress != total_size:
                raise ValueError(f"Incomplete download for {file_name}")

            elapsed_time = time.time() - start_time
            speed = (progress / (1024 * 1024)) / elapsed_time if elapsed_time > 0 else 0
            downloaded_size_gb = human_readable_size(progress)
            total_size_gb = human_readable_size(total_size)
            status = download_status.get(file_name, 'Completed')
            print(f"{file_name} \t\t\t {speed:.2f} MB/s, {downloaded_size_gb} / {total_size_gb}, {status}")
            logging.info(f"Successfully downloaded {file_name}")
            download_status[file_name] = 'Completed'
            completed_downloads += 1
            if file_name in download_speeds:
                del download_speeds[file_name]

    except Exception as e:
        logging.error(f"Error downloading {file_name}: {e}")
        download_status[file_name] = 'Error'
        if os.path.exists(file_path):
            os.remove(file_path)
        if file_name in download_speeds:
            del download_speeds[file_name]

lock = threading.Lock()

def print_progress():
    with lock:
        for file_name, status in list(download_status.items()):
            print(f"{file_name}: {status}")

        remaining_downloads = total_downloads - completed_downloads
        print(f"Completed: {completed_downloads}, Remaining: {remaining_downloads}")

def print_progress2():
    global download_speeds
    remaining_downloads = total_downloads - completed_downloads
    total_speed = sum(download_speeds.values())
    print(f"Completed: {completed_downloads}, Remaining: {remaining_downloads}, Total Speed: {total_speed:.2f} MB/s")

def print_progress_periodically():
    while True:
        print_progress2()
        time.sleep(5)
		
		
downloads = [
        ("https://civitai.com/api/download/models/245598?type=Model&format=SafeTensor&size=full&fp=fp16", "Realistic_Vision_v_6_0_B1.safetensors"),
		("https://civitai.com/api/download/models/130072?type=Model&format=SafeTensor&size=full&fp=fp16", "Realistic Vision_v_5_1.safetensors"),
        ("https://civitai.com/api/download/models/128713?type=Model&format=SafeTensor&size=pruned&fp=fp16", "DreamShaper_v_8.safetensors"),
		("https://civitai.com/api/download/models/176425", "majicMIX_realistic_v_7.safetensors"),
		("https://civitai.com/api/download/models/143906?type=Model&format=SafeTensor&size=pruned&fp=fp16", "epiCRealism_Natural_RC1_VAE.safetensors"),
		("https://civitai.com/api/download/models/46846?type=Model&format=SafeTensor&size=full&fp=fp32", "ReVAnimated_v_1_2_2.safetensors"),
		("https://civitai.com/api/download/models/119057?type=Model&format=SafeTensor&size=pruned&fp=fp16", "MeinaMix_v_11.safetensors"),
		("https://civitai.com/api/download/models/256915", "CyberRealistic_v_4_1.safetensors"),
		("https://civitai.com/api/download/models/177164", "RealisticAsians_v_7.safetensors"),
		("https://civitai.com/api/download/models/57618", "Counterfeit_v_3.safetensors"),
		("https://civitai.com/api/download/models/223670?type=Model&format=SafeTensor&size=pruned&fp=fp16", "epiCPhotoGasm_LastUnicorn.safetensors"),
		("https://civitai.com/api/download/models/201259", "epiCPhotoGasm_Z_Universal.safetensors"),
		("https://civitai.com/api/download/models/132760?type=Model&format=SafeTensor&size=pruned&fp=fp16", "AbsoluteReality_v_1_8_1.safetensors"),
		("https://civitai.com/api/download/models/76907", "GhostMix_v_2_0.safetensors"),
		("https://civitai.com/api/download/models/102222", "XXMix_9realistic_v_4_0.safetensors"),
		("https://civitai.com/api/download/models/95489?type=Model&format=SafeTensor&size=full&fp=fp16", "AnyLoRA.safetensors"),
		("https://civitai.com/api/download/models/125771", "ToonYou_Beta_6.safetensors"),
		("https://civitai.com/api/download/models/93208?type=Model&format=SafeTensor&size=pruned&fp=fp16", "DarkSushiMix_v_2_25.safetensors"),
		("https://civitai.com/api/download/models/261539", "AnalogMadness_v_7.safetensors"),
		("https://civitai.com/api/download/models/296456", "YesMix_v_4.safetensors"),
		("https://civitai.com/api/download/models/272376", "PicX_real.safetensors"),
		("https://civitai.com/api/download/models/276583", "RealCartoon3D_v_13.safetensors"),
		("https://civitai.com/api/download/models/116876", "SDXL_Unstable_Diffusers_v_11.safetensors"),
		("https://civitai.com/api/download/models/274039", "Juggernaut_Reborn.safetensors"),
		("https://civitai.com/api/download/models/113623", "LEOSAM_Film_Grain.safetensors"),
		("https://civitai.com/api/download/models/299667", "RealCartoon_Realistic_v_13.safetensors"),
		("https://civitai.com/api/download/models/296818", "AingDiffusion_v_15.safetensors"),
		("https://civitai.com/api/download/models/250801", "AniMerge_v_2_6.safetensors"),
		("https://civitai.com/api/download/models/295351", "Real_Dream_v_12.safetensors"),
		("https://civitai.com/api/download/models/270057", "CyberRealistic_Classic_v_2_1.safetensors"),
		("https://civitai.com/api/download/models/266360", "Flat_2D_Animerge_v_4_5_sharp.safetensors"),
		("https://civitai.com/api/download/models/301295", "RealCartoon_Pixar_v_8.safetensors"),
		("https://civitai.com/api/download/models/300394", "RealCartoon_Anime_v_10.safetensors"),
		("https://civitai.com/api/download/models/288888", "The_Truality_Engine_PRO.safetensors"),
		("https://civitai.com/api/download/models/266507", "HimawariMix_v_10.safetensors"),
		("https://civitai.com/api/download/models/294902", "RaemuMix_v_8_1.safetensors"),
		("https://civitai.com/api/download/models/286238", "majicMIX_lux_v_3.safetensors"),
		("https://civitai.com/api/download/models/292387", "t3_2_5_d_v_1.safetensors"),
		("https://civitai.com/api/download/models/281273", "t3_v_10.safetensors"),
		("https://civitai.com/api/download/models/265285", "Noosphere_v_4_2.safetensors"),
		("https://civitai.com/api/download/models/291921", "Blazing_Drive_v13_md.safetensors"),
		("https://civitai.com/api/download/models/299029", "Colorful_v_8.safetensors"),
		("https://civitai.com/api/download/models/268052", "SoftRealistic_v_155.safetensors"),
		("https://civitai.com/api/download/models/266702", "PVC_Style_Model_v_3.safetensors"),
		("https://civitai.com/api/download/models/255415", "Arthemy_Comics_v_6.safetensors"),
		("https://civitai.com/api/download/models/59409", "ChikMix_v_3.safetensors"),
		("https://civitai.com/api/download/models/121557?type=Model&format=SafeTensor&size=pruned&fp=fp16", "ICBINP_SECO.safetensors"),
		("https://civitai.com/api/download/models/70458", "Henmix_Real_v_4.safetensors"),
		("https://civitai.com/api/download/models/72396", "Lyriel_v_1_6.safetensors"),
		("https://civitai.com/api/download/models/64094?type=Model&format=SafeTensor&size=full&fp=fp16", "NeverEnding_Dream_NED_v_1_22.safetensors"),
		("https://civitai.com/api/download/models/106289", "GuoFeng3_v_3_4.safetensors"),
		("https://civitai.com/api/download/models/90072", "Photon_v_1.safetensors"),
		("https://civitai.com/api/download/models/238656", "DivineEleganceMix_v_9.safetensors"),
		("https://civitai.com/api/download/models/6223", "Pastel_Mix_Full.safetensors"),
		("https://civitai.com/api/download/models/99805", "A_Zovya_Photoreal_v_2.safetensors"),
		("https://civitai.com/api/download/models/112809", "MeinaUnreal_v_4_1.safetensors"),
		("https://civitai.com/api/download/models/108289?type=Model&format=SafeTensor&size=pruned&fp=fp32", "MeinaPastel_v_6_Pastel.safetensors"),
		("https://civitai.com/api/download/models/192339", "fantasticmix_k1.safetensors"),
		("https://civitai.com/api/download/models/8137", "RealDosMix.safetensors"),
		("https://civitai.com/api/download/models/146253", "LOFI_v_4.safetensors"),
		("https://civitai.com/api/download/models/108545?type=Model&format=SafeTensor&size=pruned&fp=fp16", "Mistoon_Anime_v_2.safetensors"),
		("https://civitai.com/api/download/models/116876", "Unyielding_Grimoire.safetensors"),
		("https://civitai.com/api/download/models/115942?type=Model&format=SafeTensor&size=pruned&fp=fp16", "Realisian_v_5.safetensors"),
		("https://civitai.com/api/download/models/127207?type=Model&format=SafeTensor&size=full&fp=fp32", "Juggernaut_Aftermath.safetensors"),
		("https://civitai.com/api/download/models/114770", "Juggernaut_Final.safetensors"),
		("https://civitai.com/api/download/models/221220", "TMND_Mix_SP_RAINBOW.safetensors"),
		("https://civitai.com/api/download/models/203733", "TMND_Mix_VII.safetensors"),
		("https://civitai.com/api/download/models/124626", "RPG_v_5.safetensors"),
		("https://civitai.com/api/download/models/6087", "Kotosmix_v_1.safetensors"),
		("https://civitai.com/api/download/models/69832", "Disney_Pixar_Cartoon_Type_A.safetensors"),
		("https://civitai.com/api/download/models/117136", "SDVN3_RealArt.safetensors"),
		("https://civitai.com/api/download/models/141866?type=Model&format=SafeTensor&size=pruned&fp=fp16", "Dark_Sushi_2_5D.safetensors"),
		("https://civitai.com/api/download/models/14459", "YoZoRa_V1_origin.safetensors"),
		("https://civitai.com/api/download/models/137204", "Experience_v_10.safetensors"),
		("https://civitai.com/api/download/models/17084", "KoreanStyle2_5D.safetensors"),
		("https://civitai.com/api/download/models/156263", "Ether_Real_Mix_v_3_1.safetensors"),
		("https://civitai.com/api/download/models/86329", "QGO_PromptingReal_v_1.safetensors"),
		("https://civitai.com/api/download/models/178711", "yayoi_mix_v_2_5.safetensors"),
		("https://civitai.com/api/download/models/188171", "Manmaru_mix_v_3.safetensors"),
		("https://civitai.com/api/download/models/256869", "majicMIX_fantasy_v_3.safetensors"),
		("https://civitai.com/api/download/models/5021", "AbyssOrangeMix2.safetensors"),
		("https://civitai.com/api/download/models/85614", "BeenYou_R13.safetensors"),
		("https://civitai.com/api/download/models/127416", "SXZ_Luma_0_9X.safetensors"),
		("https://civitai.com/api/download/models/237459?type=Model&format=SafeTensor&size=full&fp=fp32", "DarkSun_v_7b.safetensors"),
		("https://civitai.com/api/download/models/89927?type=Model&format=SafeTensor&size=full&fp=fp16", "AnyLoRA_Anime_Mix_v1.safetensors"),
		("https://civitai.com/api/download/models/137207", "Consistent_Factor_Euclid_v6_1.safetensors"),
		("https://civitai.com/api/download/models/236963?type=Model&format=SafeTensor&size=pruned&fp=fp16", "Realistic_Digital_Genius_v_6.safetensors"),
		("https://civitai.com/api/download/models/88738", "Vendo_Photorealistic_2.safetensors"),
		("https://civitai.com/api/download/models/198401?type=Model&format=SafeTensor&size=pruned&fp=fp16", "CyberRealistic_v_4.safetensors"),
		("https://civitai.com/api/download/models/142465?type=Model&format=SafeTensor&size=pruned&fp=fp16", "Objective_Reality_v2.safetensors"),
		("https://huggingface.co/XpucT/Deliberate/resolve/main/Deliberate_v5%20(SFW).safetensors", "Deliberate_v5.safetensors"),
		("https://huggingface.co/XpucT/Deliberate/resolve/main/Deliberate_v3.safetensors", "Deliberate_v3.safetensors"),
		("https://civitai.com/api/download/models/117986", "Samaritan_3d_Cartoon.safetensors"),
		("https://civitai.com/api/download/models/139087", "OnlyRealistic_v_30.safetensors"),
		("https://civitai.com/api/download/models/42985", "Cheese_Daddy_Landscapes_v_4_1.safetensors"),
		("https://civitai.com/api/download/models/129974", "OrangeChillMix_v_7.safetensors"),
		("https://civitai.com/api/download/models/267036", "AmIReal_v_4_3.safetensors"),
		("https://civitai.com/api/download/models/170209", "MagMix_v_9.safetensors"),
		("https://huggingface.co/runwayml/stable-diffusion-v1-5/resolve/main/v1-5-pruned.safetensors", "stable_diffusion_v_1_5.safetensors"),
		("https://civitai.com/api/download/models/258179", "amour_bold_v_3_5.safetensors"),
		("https://civitai.com/api/download/models/286170", "RealLife_v_4.safetensors"),
		("https://civitai.com/api/download/models/153135", "PicX_v_1.safetensors"),
		("https://civitai.com/api/download/models/286153", "AnReal_v_3.safetensors"),
		("https://civitai.com/api/download/models/301292", "AnRealSpiceMix_v_3.safetensors"),
		("https://civitai.com/api/download/models/192735", "Gui_Realism_Photo_v_1.safetensors"),
		("https://civitai.com/api/download/models/121220", "Opiate_v_3.safetensors"),
		("https://civitai.com/api/download/models/190121", "Stoked_Reality_v_2_2.safetensors"),
		("https://civitai.com/api/download/models/230176", "Tooneame_v_1.safetensors"),
		("https://civitai.com/api/download/models/172938", "Beyond_Reality_v_2.safetensors"),
		("https://civitai.com/api/download/models/299198", "NoosphereREAL_v_2.safetensors"),
		("https://civitai.com/api/download/models/248599", "Serenity_v_2.safetensors"),
		("https://civitai.com/api/download/models/236513", "Corbe_Realism_Mix_v_1.safetensors"),
		("https://civitai.com/api/download/models/158155", "Art_Universe_v_8.safetensors"),
		("https://civitai.com/api/download/models/255666", "JernauMix_Chimera_v_3.safetensors"),
		("https://civitai.com/api/download/models/127963", "LahMix_Mysterious_v_2.safetensors"),
		("https://civitai.com/api/download/models/252405", "AniToon_v_1_1.safetensors"),
		("https://civitai.com/api/download/models/262227", "Kawaii_Realistic_European_Mix_v_0_5.safetensors"),
		("https://civitai.com/api/download/models/261778", "JXKRE_v_1.safetensors"),
		("https://civitai.com/api/download/models/212220", "WeinComic_v_1.safetensors"),
		("https://civitai.com/api/download/models/216043", "Summershard_v_1.safetensors"),
		("https://civitai.com/api/download/models/292213", "Hyper_Realism_v_3.safetensors"),
		("https://civitai.com/api/download/models/180448", "DivineAnimeMix_v_2.safetensors"),
		("https://civitai.com/api/download/models/292264", "WildCardX_v_2.safetensors"),
		("https://civitai.com/api/download/models/264463", "BeautyFool_UltraReal_v_1.safetensors"),
		("https://civitai.com/api/download/models/224811", "SemiRealistic_v54.safetensors"),
		("https://civitai.com/api/download/models/243482", "CyberRealistic_2_5D_Style_v_1.safetensors"),
		("https://civitai.com/api/download/models/238947", "PorousRox_v_1.safetensors"),
		("https://civitai.com/api/download/models/123949", "DucHaiten_Retro_v_2.safetensors"),
		("https://civitai.com/api/download/models/230263", "Non_Plus_Ultra_v_2.safetensors"),
		("https://civitai.com/api/download/models/207008", "3D_Cartoon_Vision_v_1.safetensors"),
		("https://civitai.com/api/download/models/251207", "fennfoto_ff2.safetensors"),
		("https://civitai.com/api/download/models/107812", "blue_pencil_v_10.safetensors"),
		("https://civitai.com/api/download/models/256560", "Airtist_MAL_Realistic_v_2_5.safetensors"),
		("https://civitai.com/api/download/models/232446", "RaesanMix_v_4_1.safetensors"),
		("https://civitai.com/api/download/models/236036", "AniMics_v_1.safetensors"),
		("https://civitai.com/api/download/models/222240", "majicMIX_alpha_v_2.safetensors"),
		("https://civitai.com/api/download/models/237857", "Sals_Realism_beta_v_4.safetensors"),
		("https://civitai.com/api/download/models/131530", "NextPhoto_v_3.safetensors"),
		("https://civitai.com/api/download/models/191384", "PhotoSomnia_Omega.safetensors"),
		("https://civitai.com/api/download/models/180253", "UmbraMix_v_5.safetensors"),
		("https://civitai.com/api/download/models/257926", "helloArt_Oil_v_1.safetensors"),
		("https://civitai.com/api/download/models/96912", "WyvernMix_v_9.safetensors"),
		("https://civitai.com/api/download/models/88171", "Paragon_v_1.safetensors"),
		("https://civitai.com/api/download/models/118727", "Milky_Chicken_v_2.safetensors"),
		("https://civitai.com/api/download/models/269610", "LimitlessVision_v_3.safetensors"),
		("https://civitai.com/api/download/models/271398", "Arthemy_Real_v_1.safetensors"),
		("https://civitai.com/api/download/models/191538", "Incredible_World_v_4.safetensors"),
		("https://civitai.com/api/download/models/147577", "Realistic_Fantasy_v_2.safetensors"),
		("https://civitai.com/api/download/models/88158", "RunDiffusion_FX_Photorealistic_v_1.safetensors"),
		("https://civitai.com/api/download/models/105796", "FaeTastic_v_2.safetensors"),
		("https://civitai.com/api/download/models/296546", "Daring_Hydra_v_1_1.safetensors"),
		("https://civitai.com/api/download/models/260358", "Beauty_By_Stable_Yogi_v_3.safetensors"),
		("https://civitai.com/api/download/models/254248", "dreamyCybergasm_v_1_4.safetensors"),
		("https://civitai.com/api/download/models/195589", "hellokid25D_v_1_5.safetensors"),
		("https://civitai.com/api/download/models/177632", "HyperNatural_v_1.safetensors"),
		("https://civitai.com/api/download/models/298786", "Cornucopia_v_2_2.safetensors"),
		("https://civitai.com/api/download/models/152070", "AWPortrait_v_1_2.safetensors"),
		("https://civitai.com/api/download/models/86412", "VirtuousMix_v_1_0.safetensors"),	
		("https://civitai.com/api/download/models/287877","YACAM_SR_16111.safetensors"),
		("https://civitai.com/api/download/models/292646","Photographer_Alpha7_v_3.safetensors"),
		("https://civitai.com/api/download/models/247225","AniVerse_v_1_6.safetensors")

]		

def main():
    global total_downloads
    setup_logging()
    folder = r"stable-diffusion-webui/models/Stable-diffusion"
    if not os.path.exists(folder):
        os.makedirs(folder)

    total_downloads = len(downloads)
    update_completed_downloads(downloads, folder)
    max_threads = int(input("Enter the number of simultaneous downloads: "))
    print_progress()
    progress_thread = threading.Thread(target=print_progress_periodically, daemon=True)
    progress_thread.start()

    with ThreadPoolExecutor(max_workers=max_threads) as executor:
        future_to_download = {executor.submit(download_file, url, folder, file_name): file_name for url, file_name in downloads}
        for future in as_completed(future_to_download):
            file_name = future_to_download[future]

    print("All downloads scheduled. Exiting...")

if __name__ == "__main__":
    main()
